/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdint.h>
#include <malloc.h>
#include <string.h>

void addRom(char *pName, uint8_t *pBuff, uint32_t pos, uint32_t romSize);

int main(int argc, char* argv[])
{
	uint32_t romSize = 1024 * 1024;
	uint8_t *pBuff;
	char *at;
	uint32_t a, pos, written;

	pBuff = (uint8_t*)malloc(romSize);
	memset(pBuff, 0xFF, romSize);

	a = 1;

	while(a < argc)
	{
		pos = 0;
		if((at = strchr(argv[a], '@')) != NULL)
		{
			*at++ = 0;
			if(at[0] == '0' && at[1] == 'x')
			{
				sscanf(at, "0x%x", &pos);
			}
			else
			{
				sscanf(at, "%u", &pos);
			}
		}

		fprintf(stderr, "ROM %s added at 0x%X \n", argv[a], pos);
		addRom(argv[a], pBuff, pos, romSize);
		a++;
	}

	while(romSize > 0)
	{
		written = fwrite(pBuff, 1, romSize > 1024 ? 1024 : romSize, stdout);
		pBuff += written;
		romSize -= written;
	}
}

void addRom(char *pName, uint8_t *pBuff, uint32_t pos, uint32_t romSize)
{
	FILE *f;
	uint32_t size, read;

	f = fopen(pName, "rb");
	if(!f)
	{
		fprintf(stderr, "Couldn't find file %s\n", pName);
		return;
	}

	fseek(f, 0, SEEK_END);
	size = ftell(f);
	fseek(f, 0, SEEK_SET);

	if(pos + size > romSize)
	{
		fprintf(stderr, "ROM file %s will be cropped since it goes over the end\n", pName);
		size = romSize - pos;
	}

	pBuff += pos;

	while(size > 0)
	{
		read = fread(pBuff, 1, size > 1024 ? 1024 : size, f);
		size -= read;
		pBuff += read;
	}

	fclose(f);
	
}
